/*
    This file is part of lorcon

    lorcon is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    lorcon is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with lorcon; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

    Copyright (c) 2005 dragorn and Joshua Wright
*/

#ifndef __TX80211_H__
#define __TX80211_H__

#include <stdint.h>
#include <errno.h>
#include <string.h>
#include <stdio.h>
#include <unistd.h>

#include <tx80211_packet.h>

/* Error string length */
#define TX80211_STATUS_MAX		1024

#define MAX_IFNAME_LEN			32

/* Master tx80211 interface structure */
struct tx80211 {
	/* 
	 * Internal functions and structs 
	 *
	 * These should not be called directly.
	 */

	char ifname[MAX_IFNAME_LEN];
	uint32_t capabilities;
	int raw_fd;
	int ioctl_fd;
	int packets_sent;
	int packets_recv;

    int mode;
    int channel;

	/* Extra space for use by the driver code */
	void *extra;
	
	/* Callthroughs to functions installed by the setup when the injector is
	 * defined. */
	int (*open_callthrough)(struct tx80211 *);
	int (*close_callthrough)(struct tx80211 *);

	int (*setmode_callthrough)(struct tx80211 *, int);
	int (*getmode_callthrough)(struct tx80211 *);

	int (*setfuncmode_callthrough)(struct tx80211 *, int);

	int (*setchan_callthrough)(struct tx80211 *, int);
	int (*getchan_callthrough)(struct tx80211 *);

	int (*txpacket_callthrough)(struct tx80211 *, struct tx80211_packet *);
};

/* Dynamic structure of supported cards, for clients to list supported types */
struct tx80211_cardlist {
	char **cardnames;
	char **descriptions;
	int *capabilities;
	int num_cards;
};


/* tx80211 device capabilities */
#define TX80211_CAP_NONE			0	/* no capabilities */
#define TX80211_CAP_SNIFF			1	/* capable of standard rfmon sniffing */
#define TX80211_CAP_TRANSMIT		2	/* capable of basic transmitting */
#define TX80211_CAP_SEQ				4	/* capable of sequence spoofing */
#define TX80211_CAP_BSSTIME			8	/* capable of bss timestamp spoofing */
#define TX80211_CAP_PHY				16	/* capable of phy-frame spoofing */
#define TX80211_CAP_FRAG            32  /* capable of sending fragments */
#define TX80211_CAP_CTRL            64  /* capable of sending ctrl frames */
#define TX80211_CAP_DURID          128  /* capable of setting DUR/ID field */

/* 
 * tx802.11 functions for creating and operating on the tx80211 struct 
 * core API of library
 */

/* Injector types */
#define INJ_NODRIVER    0
#define INJ_WLANNG  	1
#define INJ_HOSTAP  	2
#define INJ_AIRJACK 	3
#define INJ_PRISM54 	4
#define INJ_MADWIFI 	5
#define INJ_RTL8180     6
#define INJ_RT2500		7
#define INJ_MAX         8

/* Populate and destroy lists of available tx types 
 * Caller of freecardlist must be aware that in_list is destroyed! */
struct tx80211_cardlist *tx80211_getcardlist();
void tx80211_freecardlist(struct tx80211_cardlist *in_list);

/* Resolve a card "name" to a card number */
int tx80211_resolvecard(const char *in_str);

/* Create and assign a tx80211 system */
int tx80211_init(struct tx80211 *in_tx, char *in_ifname, int in_injector);

/* Get the capabilities */
int tx80211_getcapabilities(struct tx80211 *in_tx);

/* Open and close an interface */
int tx80211_open(struct tx80211 *in_tx);
int tx80211_close(struct tx80211 *in_tx);

/* Get and set modes */
int tx80211_setmode(struct tx80211 *in_tx, int in_mode);
int tx80211_getmode(struct tx80211 *in_tx);

/* Set "functional" mode ("tx+monitor", etc, vs. absolute mode) */
int tx80211_setfunctionalmode(struct tx80211 *in_tx, int in_fmode);

/* Get and set channels */
int tx80211_setchannel(struct tx80211 *in_tx, int in_chan);
int tx80211_getchannel(struct tx80211 *in_tx);

/* Transmit a frame */
int tx80211_txpacket(struct tx80211 *in_tx, struct tx80211_packet *in_packet);

#endif

